#!/usr/bin/env python
# -- coding: utf8 --

#Copyright (C) 2007   Authors: Jacobo Tarragón Cros, Víctor Muñoz Ferrara, William Viana Soares. 
#Email addresses: tarragon[AT]aditel[DOT]org, victorm[AT]aditel[DOT]org, wviana[AT]aditel[DOT]org
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.


import os
import sys
import random
import ID3
import config

DATAPATH = os.path.join(os.path.expanduser('~'), '.musicquizz') #/home/user/.musicquizz

class Preferencias:
	def __init__(self, fichero=os.path.join(DATAPATH, "default.conf")):
		
		self.fichero = fichero
		if not os.path.exists(fichero):
			self.generarPreferencias(fichero)
		f = file(fichero)
		#config.Config.__init__(self, f, parent=config.Mapping)
		self.ops = config.Config(f)
		
	def recargarPreferencias(self):
		f = file(self.fichero)
		self.ops = config.Config(f)

	def generarPreferencias(self, fich):
		f = open(fich,"w")
		f.write("directorios : []\n")
		f.write("rondas : 50\n")
		f.write("opciones : 4\n")
		f.write("tiempo : 10\n")
		f.write("titulo : True\n")
		f.write("autor : True\n")
		f.write("fichero : '%s'" % os.path.join(DATAPATH, 'canciones.dat'))
		f.close()
	
	def guardar(self):
		f = open(self.fichero, "w")
		self.ops.save(f)
		f.close()
		
	def __str__(self):
		return "Fichero: " + str(self.ops.fichero) + "\n" +\
		"Directorios: " + str(self.ops.directorios) + "\n" +\
		"Tiempo de ronda: " + str(self.ops.tiempo) + "\n" +\
		"Número de opciones: " + str(self.ops.opciones) + "\n" +\
		"Número de rondas: " + str(self.ops.rondas) + "\n" +\
		"Mostrar título: " + str(self.ops.titulo) + "\n" +\
		"Mostrar autor: " + str(self.ops.autor)
		
	
class Cancion:
	def __init__(self, ruta, titulo, autor):
		self.ruta = ruta
		self.titulo = titulo
		self.autor = autor
		
	def __str__(self):
		return "Ruta: " + self.ruta +"\n" +\
			"Titulo: " + self.titulo + "\n"+\
			"Autor: " + self.autor		
class Coleccion:
	""" Clase para gestionar la colección de música"""
	
	def __init__(self, preferencias):
		self.idx_cancion_actual = -1
		self.lista_canciones = []
		if not os.path.exists(preferencias.ops.fichero):
			print "[MusicQuizzCore] Explorando colección..."
			self.explorarColeccion(preferencias.ops.directorios, preferencias.ops.fichero)
			print "[MusicQuizzCore] Exploración completa"
		self.crearListaCanciones(preferencias.ops.fichero)
		self.num_canciones = len(self.lista_canciones)
		#self.validarNumCanciones(preferencias.ops.opciones)
			
	def crearListaCanciones(self, fich_canciones):
		""" Crea una lista de canciones a partir del fichero de información de la colección
		y la desordena.
		"""
		
		if not os.path.exists(fich_canciones):
			raise IOError, "No se encuentra el fichero de canciones"
		canciones = open(fich_canciones)
		for cancion in canciones:
			cancion = cancion.split("|")
			self.lista_canciones.append(Cancion(cancion[0],cancion[1],cancion[2]))
		canciones.close()
		
		random.shuffle(self.lista_canciones)
		
	def validarNumCanciones(self, num_opciones):
		"""
		Cuando se empieza a jugar sin canciones válidas en los directorios, abrir directamente el diálogo de configuración para que el usuario las añada.
		"""
		if self.num_canciones < num_opciones:
			print "[MusicQuizzCore] Aviso: no se han encontrado ficheros mp3 válidos en ningún directorio "
			sys.exit(1)
			
	def crearFicheroCanciones(self, paths, nom_fich):
		""" Crea un fichero a partir de una lista de directorios que contienen archivos mp3.
		Ese fichero contiene en cada línea los siguientes campos separados por | :
			ruta|titulo|autor 
		"""
			
		n_canciones = 0
		fich = open(nom_fich,"w")
		
		for dire in paths:
			if not os.path.exists(dire):
				print "[MusicQuizzCore] Aviso: no se ha podido encontrar el directorio ",dire
				continue
			for raiz, aux,ficheros in os.walk(dire):
				for fichero in ficheros:
					ruta = raiz+'/'+fichero
					try:
						id=ID3.ID3(ruta)
						if id.has_key("TITLE") and id.has_key("ARTIST"):
							fich.write(ruta+'|'+id["TITLE"]+'|'+id["ARTIST"]+'\n')
							n_canciones+=1
					except:
						pass
		fich.close()
		
		return n_canciones

	def explorarColeccion(self, paths, nom_fich):
		""" Crea el fichero de información y a continuación lo carga en la lista """
		self.crearFicheroCanciones(paths, nom_fich)
		self.crearListaCanciones(nom_fich)
		self.num_canciones = len(self.lista_canciones)
		
	def pop(self):
		self.idx_cancion_actual += 1
		
		#si se ha llegado al final de la lista, empezar del principio y barajar
		if self.idx_cancion_actual == self.num_canciones:
			self.idx_cancion_actual = 0
			random.shuffle(self.lista_canciones)
		return self.idx_cancion_actual
	
	def __getitem__(self,indice):
		if indice < 0 or indice >= self.num_canciones:
			raise IndexError, "fuera de rango"
		return self.lista_canciones[indice]
	
	def __len__(self):
		return self.num_canciones
	
class Juego:
	def __init__(self):
		self.prefs = Preferencias(os.path.join(DATAPATH, "default.conf"))
		self.coleccion = Coleccion(self.prefs)
		self.puntuacion = 0
		self.ronda = 1
		self.aciertos = 0        
		self.idx_verdadera = 0          #indice de la canción en la colección 
		self.canciones_juego = [] 
		self.verdadera = 0              #indice de la canción en la lista actual de juego
		
	def prepararRonda(self):
		
		verdadera = self.sacarCancionVerdadera()
		self.canciones_juego = [verdadera] + self.sacarCandidatas()
		self.idx_verdadera = verdadera
		random.shuffle(self.canciones_juego)
		#averiguar en qué posición de la lista se ha quedado la verdadera después del shuffle
		i = 0
		while self.canciones_juego[i] != self.idx_verdadera:
			i+=1
		self.verdadera = i
		return [self.coleccion[i] for i in self.canciones_juego]

	def finRonda(self, idx, tiempo):
		"""Devuelve True si aún quedan rondas por jugar."""
		if self.comprobarAcierto(idx):
			self.sumarPuntuacion(tiempo)
			self.aciertos += 1
		return self.incrementarRonda()
		
	
	def reset(self):
		self.puntuacion = 0
		self.aciertos = 0
		self.ronda = 1
		
	def sacarCancionVerdadera(self):
		return self.coleccion.pop()
		
	def sacarCandidatas(self):
		idx_candidatas = []
		for i in range(self.prefs.ops.opciones-1):
			n = random.randint(0, len(self.coleccion)-1)
			while(n == self.idx_verdadera or n in idx_candidatas):
				n = random.randint(0,len(self.coleccion)-1)
			idx_candidatas.append(n)
		return [idx for idx in idx_candidatas]
	
	def comprobarAcierto(self, idx):
		return idx != None and self.canciones_juego[idx] == self.idx_verdadera
	
	def incrementarRonda(self):
		if self.ronda == self.prefs.ops.rondas:
			return False
		self.ronda += 1
		return True
	
	def sumarPuntuacion(self, tiempo):
		puntos = int(100 * tiempo)
		self.puntuacion += puntos
	
	def strPuntos(self):
		return str(self.puntuacion)
	
	def strRonda(self):
		return str(self.ronda) + " / " + str(self.prefs.ops.rondas)
	
	def promedioAciertos(self, fin=False):
		if not fin:
			return str(self.aciertos) + " / " + str(self.ronda-1)
		else:
			return str(self.aciertos) + " / " + str(self.ronda)
			
	
if __name__ == "__main__":
	
	juego = Juego()
	
	print "CONFIGURACIÓN ACTUAL"
	print juego.prefs

	# for i in range(10):
		# juego.sacarCancionesJuego()
		# print "Verdadera: \n",juego.coleccion[juego.idx_verdadera]
	#for cancion in juego.canciones_juego:
		#print juego.coleccion[cancion]
